//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: IcaPhysicsList.cc,v 1.7 2002/01/09 17:24:20 ranjard Exp $
// GEANT4 tag $Name: geant4-04-00-patch-02 $
//
// 

#include "G4Timer.hh"

#include "IcaPhysicsList.hh"
#include "IcaDetectorConstruction.hh"
#include "IcaPhysicsListMessenger.hh"

#include "IcaAnalysisManager.hh"

#include "globals.hh"
#include "G4ParticleDefinition.hh"
#include "G4ParticleWithCuts.hh"
#include "G4ProcessManager.hh"
#include "G4ProcessVector.hh"
#include "G4ParticleTypes.hh"
#include "G4ParticleTable.hh"
#include "G4Material.hh"
#include "G4MaterialTable.hh"
#include "G4ios.hh"
#include "iomanip"   

#include "G4FastSimulationManagerProcess.hh"

///////////////////////////////////////////////////////////////////////////

IcaPhysicsList::IcaPhysicsList(IcaDetectorConstruction* IcaDC)
:G4VUserPhysicsList(), MaxChargedStep(DBL_MAX), theeminusStepCut(0),
 theeplusStepCut(0), themuStepCut(0), thehadronStepCut(0),
 theCerenkovProcess(0), theScintillationProcess(0), pDet(IcaDC)
{
  if (!IcaDC)
    {
      G4cerr << "Warning: IcaPhysicsList default constructor called !!!"
	     << G4endl;
    }

  currentDefaultCut = defaultCutValue = 1.0 * mm ;

  cutForGamma = defaultCutValue ;
  cutForElectron = defaultCutValue ;
  cutForMuon = defaultCutValue ;
  cutForProton = defaultCutValue ;
  cutForOpticalPhoton = defaultCutValue ;

  MaxNumOfCerenkovPhotonsPerStep = 1000;

  ScintillationYieldFactor = 1.0;

  SetVerboseLevel(1);
  physicsListMessenger = new IcaPhysicsListMessenger(this);
}

///////////////////////////////////////////////////////////////////////////

IcaPhysicsList::~IcaPhysicsList()
{
  delete physicsListMessenger;
}

///////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::ConstructParticle()
{
  // In this method, static member functions should be called
  // for all particles which you want to use.
  // This ensures that objects of these particle types will be
  // created in the program. 

  ConstructBosons();
  ConstructLeptons();
  ConstructMesons();
  ConstructBaryons();
  ConstructIons();

  // Histogramming
  if (gIcaAnalysisManager) gIcaAnalysisManager->ConstructParticle();
}

///////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::ConstructBosons()
{
  // pseudo-particles
  G4Geantino::GeantinoDefinition();
  G4ChargedGeantino::ChargedGeantinoDefinition();

  // gamma
  G4Gamma::GammaDefinition();

  // optical photon
  G4OpticalPhoton::OpticalPhotonDefinition();
}

///////////////////////////////////////////////////////////////////////////

#include "G4LeptonConstructor.hh"
void IcaPhysicsList::ConstructLeptons()
{
  // Construct all leptons
  G4LeptonConstructor pConstructor;
  pConstructor.ConstructParticle();
}

///////////////////////////////////////////////////////////////////////////

#include "G4MesonConstructor.hh"
void IcaPhysicsList::ConstructMesons()
{
  //  Construct all mesons
  G4MesonConstructor pConstructor;
  pConstructor.ConstructParticle();
}

///////////////////////////////////////////////////////////////////////////

#include "G4BaryonConstructor.hh"
void IcaPhysicsList::ConstructBaryons()
{
  //  Construct all barions
  G4BaryonConstructor  pConstructor;
  pConstructor.ConstructParticle();  
}

///////////////////////////////////////////////////////////////////////////

#include "G4IonConstructor.hh"
void IcaPhysicsList::ConstructIons()
{
  //  Construct light ions
  G4IonConstructor pConstructor;
  pConstructor.ConstructParticle();  
}

///////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::ConstructProcess()
{
  AddTransportation();
  AddParameterisation();

  ConstructEM();
  ConstructOp();

  AddMaxChargedStepCut();

  ConstructGeneral();

  // Histogramming
  if (gIcaAnalysisManager) gIcaAnalysisManager->ConstructProcess();
}

///////////////////////////////////////////////////////////////////////////

#include "G4StepLimiter.hh"
#include "IcaMaxTimeCuts.hh"
#include "IcaMinEkineCuts.hh"
#include "IcaMinRangeCuts.hh"

void IcaPhysicsList::AddTransportation()
{
  G4VUserPhysicsList::AddTransportation();

  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4ProcessManager* pmanager = particle->GetProcessManager();
    pmanager->AddProcess(new G4StepLimiter());
    pmanager->AddDiscreteProcess(new IcaMaxTimeCuts());
    pmanager->AddDiscreteProcess(new IcaMinEkineCuts());
    pmanager->AddDiscreteProcess(new IcaMinRangeCuts());
  }
}

///////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::AddParameterisation()
{
  G4FastSimulationManagerProcess* 
    theFastSimulationManagerProcess = 
      new G4FastSimulationManagerProcess();
  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4ProcessManager* pmanager = particle->GetProcessManager();
    // both postStep and alongStep action are required: because
    // of the use of ghost volumes. If no ghost, the postStep
    // is sufficient.
    pmanager->AddProcess(theFastSimulationManagerProcess, -1, 1, 1);
  }
}

///////////////////////////////////////////////////////////////////////////

#include "G4Decay.hh"
void IcaPhysicsList::ConstructGeneral()
{
  // Add Decay Process
  G4Decay* theDecayProcess = new G4Decay();
  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4ProcessManager* pmanager = particle->GetProcessManager();
    if (theDecayProcess->IsApplicable(*particle)) { 
      pmanager ->AddProcess(theDecayProcess);
      // set ordering for PostStepDoIt and AtRestDoIt
      pmanager ->SetProcessOrdering(theDecayProcess, idxPostStep);
      pmanager ->SetProcessOrdering(theDecayProcess, idxAtRest);
    }
  }  
}

///////////////////////////////////////////////////////////////////////////

#include "G4ComptonScattering.hh"
#include "G4GammaConversion.hh"
#include "G4PhotoElectricEffect.hh"

#include "G4MultipleScattering.hh"

#include "G4eIonisation.hh"
#include "G4eBremsstrahlung.hh"
#include "G4eplusAnnihilation.hh"

#include "G4MuIonisation.hh"
#include "G4MuBremsstrahlung.hh"
#include "G4MuPairProduction.hh"

#include "G4hIonisation.hh"

void IcaPhysicsList::ConstructEM()
{
  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4ProcessManager* pmanager = particle->GetProcessManager();
    G4String particleName = particle->GetParticleName();
     
    if (particleName == "gamma") {
    // gamma
      // Construct processes for gamma
      pmanager->AddDiscreteProcess(new G4GammaConversion());
      pmanager->AddDiscreteProcess(new G4ComptonScattering());      
      pmanager->AddDiscreteProcess(new G4PhotoElectricEffect());

    } else if (particleName == "e-") {
    //electron
      // Construct processes for electron
      G4VProcess* theeminusMultipleScattering = new G4MultipleScattering();
      G4VProcess* theeminusIonisation = new G4eIonisation();
      G4VProcess* theeminusBremsstrahlung = new G4eBremsstrahlung();
      // add processes
      pmanager->AddProcess(theeminusMultipleScattering);
      pmanager->AddProcess(theeminusIonisation);
      pmanager->AddProcess(theeminusBremsstrahlung);      
      // set ordering for AlongStepDoIt
      pmanager->SetProcessOrdering(theeminusMultipleScattering, idxAlongStep,  1);
      pmanager->SetProcessOrdering(theeminusIonisation, idxAlongStep,  2);
      // set ordering for PostStepDoIt
      pmanager->SetProcessOrdering(theeminusMultipleScattering, idxPostStep, 1);
      pmanager->SetProcessOrdering(theeminusIonisation, idxPostStep, 2);
      pmanager->SetProcessOrdering(theeminusBremsstrahlung, idxPostStep, 3);

    } else if (particleName == "e+") {
    //positron
      // Construct processes for positron
      G4VProcess* theeplusMultipleScattering = new G4MultipleScattering();
      G4VProcess* theeplusIonisation = new G4eIonisation();
      G4VProcess* theeplusBremsstrahlung = new G4eBremsstrahlung();
      G4VProcess* theeplusAnnihilation = new G4eplusAnnihilation();
      // add processes
      pmanager->AddProcess(theeplusMultipleScattering);
      pmanager->AddProcess(theeplusIonisation);
      pmanager->AddProcess(theeplusBremsstrahlung);
      pmanager->AddProcess(theeplusAnnihilation);
      // set ordering for AtRestDoIt
      pmanager->SetProcessOrderingToFirst(theeplusAnnihilation, idxAtRest);
      // set ordering for AlongStepDoIt
      pmanager->SetProcessOrdering(theeplusMultipleScattering, idxAlongStep,  1);
      pmanager->SetProcessOrdering(theeplusIonisation, idxAlongStep,  2);
      // set ordering for PostStepDoIt
      pmanager->SetProcessOrdering(theeplusMultipleScattering, idxPostStep, 1);
      pmanager->SetProcessOrdering(theeplusIonisation, idxPostStep, 2);
      pmanager->SetProcessOrdering(theeplusBremsstrahlung, idxPostStep, 3);
      pmanager->SetProcessOrdering(theeplusAnnihilation, idxPostStep, 4);

    } else if( particleName == "mu+" || 
               particleName == "mu-"    ) {
    //muon  
     // Construct processes for muon+-
     G4VProcess* aMultipleScattering = new G4MultipleScattering();
     G4VProcess* aBremsstrahlung = new G4MuBremsstrahlung();
     G4VProcess* aPairProduction = new G4MuPairProduction();
     G4VProcess* anIonisation = new G4MuIonisation();
     // add processes
     pmanager->AddProcess(anIonisation);
     pmanager->AddProcess(aMultipleScattering);
     pmanager->AddProcess(aBremsstrahlung);
     pmanager->AddProcess(aPairProduction);
     // set ordering for AlongStepDoIt
     pmanager->SetProcessOrdering(aMultipleScattering, idxAlongStep,  1);
     pmanager->SetProcessOrdering(anIonisation, idxAlongStep,  2);
     // set ordering for PostStepDoIt
     pmanager->SetProcessOrdering(aMultipleScattering, idxPostStep, 1);
     pmanager->SetProcessOrdering(anIonisation, idxPostStep, 2);
     pmanager->SetProcessOrdering(aBremsstrahlung, idxPostStep, 3);
     pmanager->SetProcessOrdering(aPairProduction, idxPostStep, 4);

   } else if ((!particle->IsShortLived()) &&
	      (particle->GetPDGCharge() != 0.0) && 
	      (particle->GetParticleName() != "chargedgeantino")) {
     // all others charged particles except geantino
     G4VProcess* aMultipleScattering = new G4MultipleScattering();
     G4VProcess* anIonisation = new G4hIonisation();
     // add processes
     pmanager->AddProcess(anIonisation);
     pmanager->AddProcess(aMultipleScattering);
     // set ordering for AlongStepDoIt
     pmanager->SetProcessOrdering(aMultipleScattering, idxAlongStep,  1);
     pmanager->SetProcessOrdering(anIonisation, idxAlongStep,  2);
     // set ordering for PostStepDoIt
     pmanager->SetProcessOrdering(aMultipleScattering, idxPostStep, 1);
     pmanager->SetProcessOrdering(anIonisation, idxPostStep, 2);

    }
  }
}

///////////////////////////////////////////////////////////////////////////

#include "G4Cerenkov.hh"
#include "G4Scintillation.hh"
#include "G4OpAbsorption.hh"
#include "G4OpRayleigh.hh"
#include "G4OpBoundaryProcess.hh"

void IcaPhysicsList::ConstructOp()
{
  theCerenkovProcess = new G4Cerenkov("Cerenkov");
  theScintillationProcess = new G4Scintillation("Scintillation");
  G4OpAbsorption* theAbsorptionProcess = new G4OpAbsorption();
  G4OpRayleigh*   theRayleighScatteringProcess = new G4OpRayleigh();
  G4OpBoundaryProcess* theBoundaryProcess = new G4OpBoundaryProcess();

  //  theCerenkovProcess->DumpPhysicsTable();
  //  theScintillationProcess->DumpPhysicsTable();
  //  theAbsorptionProcess->DumpPhysicsTable();
  //  theRayleighScatteringProcess->DumpPhysicsTable();

#if 0
  theCerenkovProcess->SetVerboseLevel(1);
  theScintillationProcess->SetVerboseLevel(1);
  theAbsorptionProcess->SetVerboseLevel(1);
  theRayleighScatteringProcess->SetVerboseLevel(1);
  theBoundaryProcess->SetVerboseLevel(1);
#else // if 0 or 1
  theCerenkovProcess->SetVerboseLevel(0);
  theScintillationProcess->SetVerboseLevel(0);
  theAbsorptionProcess->SetVerboseLevel(0);
  theRayleighScatteringProcess->SetVerboseLevel(0);
  theBoundaryProcess->SetVerboseLevel(0);
#endif // if 0 or 1

  theCerenkovProcess->SetTrackSecondariesFirst(true);
  theCerenkovProcess->SetMaxNumPhotonsPerStep(MaxNumOfCerenkovPhotonsPerStep);

  theScintillationProcess->SetTrackSecondariesFirst(true);
  theScintillationProcess->SetScintillationYieldFactor(ScintillationYieldFactor);

  G4OpticalSurfaceModel themodel = unified;
  theBoundaryProcess->SetModel(themodel);

  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4ProcessManager* pmanager = particle->GetProcessManager();
    G4String particleName = particle->GetParticleName();
    if ((!particle->IsShortLived()) &&
	(particle->GetParticleName() != "geantino") &&
	(particle->GetParticleName() != "chargedgeantino")) {
      if (theCerenkovProcess->IsApplicable(*particle)) {
	pmanager->AddContinuousProcess(theCerenkovProcess, ordLast);
      }
      if (theScintillationProcess->IsApplicable(*particle)) {
	pmanager->AddProcess(theScintillationProcess);
	pmanager->SetProcessOrderingToLast(theScintillationProcess, idxAtRest);
	pmanager->SetProcessOrderingToLast(theScintillationProcess, idxPostStep);
      }
    }
    if (particleName == "opticalphoton") {
      pmanager->AddDiscreteProcess(theAbsorptionProcess);
      pmanager->AddDiscreteProcess(theRayleighScatteringProcess);
      pmanager->AddDiscreteProcess(theBoundaryProcess);
    }
  }
}

///////////////////////////////////////////////////////////////////////////

#include "IcaStepCut.hh"

void IcaPhysicsList::AddMaxChargedStepCut()
{
  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4ProcessManager* pmanager = particle->GetProcessManager();
    G4String particleName = particle->GetParticleName();

    if (particleName == "e-") {
      //electron
      theeminusStepCut = new IcaStepCut("ElectronStepCut");
      pmanager->AddDiscreteProcess(theeminusStepCut, ordLast);
      theeminusStepCut->SetMaxStep(MaxChargedStep);

    } else if (particleName == "e+") {
      //positron
      theeplusStepCut = new IcaStepCut("PositronStepCut");
      pmanager->AddDiscreteProcess(theeplusStepCut, ordLast);
      theeplusStepCut->SetMaxStep(MaxChargedStep);

    } else if( particleName == "mu+" || 
               particleName == "mu-"    ) {
      //muon  
     themuStepCut = new IcaStepCut("MuonStepCut");
     pmanager->AddDiscreteProcess(themuStepCut, ordLast);
     themuStepCut->SetMaxStep(MaxChargedStep);

   } else if ((!particle->IsShortLived()) &&
	      (particle->GetPDGCharge() != 0.0) && 
	      (particle->GetParticleName() != "chargedgeantino")) {
     // all others charged particles except geantino
     thehadronStepCut = new IcaStepCut("HadronStepCut");
     pmanager->AddDiscreteProcess(thehadronStepCut, ordLast);
     thehadronStepCut->SetMaxStep(MaxChargedStep);

    }
  }
}

///////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetCuts()
{
  G4Timer theTimer ;
  theTimer.Start() ;

  if (verboseLevel>0)
    {
      G4cout << "IcaPhysicsList::SetCuts:";
      G4cout << " DefaultCutLength = " << G4BestUnit(defaultCutValue,"Length") << G4endl;
    }

  // reactualise cutValues
  if (currentDefaultCut != defaultCutValue)
    {
      if(cutForGamma    == currentDefaultCut) cutForGamma    = defaultCutValue;
      if(cutForElectron == currentDefaultCut) cutForElectron = defaultCutValue;
      if(cutForMuon     == currentDefaultCut) cutForMuon     = defaultCutValue;
      if(cutForProton   == currentDefaultCut) cutForProton   = defaultCutValue;
      if(cutForOpticalPhoton == currentDefaultCut) cutForOpticalPhoton = defaultCutValue;
      currentDefaultCut = defaultCutValue;
    }

  //  " G4VUserPhysicsList::SetCutsWithDefault" method sets
  //   the default cut value for all particle types
  SetCutsWithDefault();

  // set cut values for gamma at first and for e- second and next for e+,
  // because some processes for e+/e- need cut values for gamma

  if (verboseLevel>0)
    G4cout << "IcaPhysicsList::SetCuts: GammaRangeCut = " << cutForGamma/mm << " mm" << G4endl;
  SetCutValue(cutForGamma,"gamma");

  if (verboseLevel>0)
    G4cout << "IcaPhysicsList::SetCuts: ElectronRangeCut = " << cutForElectron/mm << " mm" << G4endl;
  SetCutValue(cutForElectron,"e-");
  SetCutValue(cutForElectron,"e+");

  if (verboseLevel>0)
    G4cout << "IcaPhysicsList::SetCuts: MuonRangeCut = " << cutForMuon/mm << " mm" << G4endl;
  SetCutValue(cutForMuon,"mu-");
  SetCutValue(cutForMuon,"mu+");

  // set cut values for proton and anti_proton before all other hadrons
  // because some processes for hadrons need cut values for proton/anti_proton 

  if (verboseLevel>0)
    G4cout << "IcaPhysicsList::SetCuts: ProtonRangeCut = " << cutForProton/mm << " mm" << G4endl;
  SetCutValue(cutForProton, "proton");
  SetCutValue(cutForProton, "anti_proton");

  if (verboseLevel>0)
    G4cout << "IcaPhysicsList::SetCuts: OpticalPhotonRangeCut = " << cutForOpticalPhoton/mm << " mm" << G4endl;
  SetCutValue(cutForOpticalPhoton,"opticalphoton");

  if (verboseLevel>1) DumpCutValuesTable();

  theTimer.Stop();
  G4cout.precision(6);
  if (verboseLevel>1)
    G4cout << "Total time (SetCuts) = " << theTimer.GetUserElapsed() << " s " <<G4endl;

  // Histogramming
  if(gIcaAnalysisManager) gIcaAnalysisManager->SetCuts();
}

///////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::RestoreDefaultCuts(G4double val)
{
  if (val > 0.0) defaultCutValue = val;
  
  currentDefaultCut = defaultCutValue;
  
  cutForGamma = currentDefaultCut;
  cutForElectron = currentDefaultCut;
  cutForMuon = currentDefaultCut;
  cutForProton = currentDefaultCut;
  cutForOpticalPhoton = currentDefaultCut;
  
  SetMaxStep(-1.0);
  
  // if (pDet) pDet->ResetUserLimits();
}

///////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetGammaCut(G4double val)
{
  cutForGamma = val;
}

///////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetElectronCut(G4double val)
{
  cutForElectron = val;
}

///////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetMuonCut(G4double val)
{
  cutForMuon = val;
}

//////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetProtonCut(G4double val)
{
  cutForProton = val;
}

//////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetOpticalPhotonCut(G4double val)
{
  cutForOpticalPhoton = val;
}

////////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetCutsByEnergy(G4double val)
{
  if (!pDet)
    {
      G4cerr << "Error: IcaPhysicsList::SetCutsByEnergy : no detector !!!"
	     << G4endl;
      return;
    }

  G4ParticleTable* theParticleTable =  G4ParticleTable::GetParticleTable();
  G4Material* currMat = pDet->GetAbsorberMaterial();

  // set cut values for gamma at first and for e- second and next for e+,
  // because some processes for e+/e- need cut values for gamma

  G4ParticleDefinition* part;
  G4double cut;

#if 0
  part = theParticleTable->FindParticle("gamma");
  cut = G4EnergyLossTables::GetRange(part,val,currMat);
  SetCutValue(cut, "gamma");
#endif

  part = theParticleTable->FindParticle("e-");
  cut = G4EnergyLossTables::GetRange(part,val,currMat);
  SetElectronCut(cut); // SetCutValue(cut, "e-");

#if 0
  part = theParticleTable->FindParticle("e+");
  cut = G4EnergyLossTables::GetRange(part,val,currMat);
  SetCutValue(cut, "e+");
#endif

  part = theParticleTable->FindParticle("mu-");
  cut = G4EnergyLossTables::GetRange(part,val,currMat);
  SetMuonCut(cut); // SetCutValue(cut, "mu-");

#if 0
  part = theParticleTable->FindParticle("mu+");
  cut = G4EnergyLossTables::GetRange(part,val,currMat);
  SetCutValue(cut, "mu+");
#endif

  // set cut values for proton and anti_proton before all other hadrons
  // because some processes for hadrons need cut values for proton/anti_proton

  part = theParticleTable->FindParticle("proton");
  cut = G4EnergyLossTables::GetRange(part,val,currMat);
  SetProtonCut(cut); // SetCutValue(cut, "proton");

#if 0
  part = theParticleTable->FindParticle("anti_proton");
  cut = G4EnergyLossTables::GetRange(part,val,currMat);
  SetCutValue(cut, "anti_proton");
#endif

  SetGammaCut(cut); SetOpticalPhotonCut(cut);
  defaultCutValue = cut;

#if 0
  if (verboseLevel>1)     DumpCutValuesTable();
#endif

}

//////////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::GetRange(G4double val)
{
  if (!pDet)
    {
      G4cerr << "Error: IcaPhysicsList::GetRange : no detector !!!"
	     << G4endl;
      return;
    }

  G4ParticleTable* theParticleTable =  G4ParticleTable::GetParticleTable();
  G4Material* currMat = pDet->GetAbsorberMaterial();

  G4ParticleDefinition* part;
  G4double cut;
  part = theParticleTable->FindParticle("mu-");
  cut = G4EnergyLossTables::GetRange(part,val,currMat);
  G4cout << "material : " << currMat->GetName() << G4endl;
  G4cout << "particle : " << part->GetParticleName() << G4endl;
  G4cout << "energy   : " << val / GeV << " (GeV)" << G4endl;
  G4cout << "range    : " << cut / mm << " (mm)" << G4endl;
}

////////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetMaxStep(G4double step)
{
  if (step <= 0.0) MaxChargedStep = DBL_MAX;
  else MaxChargedStep = step ;

  if (theeminusStepCut) theeminusStepCut->SetMaxStep(MaxChargedStep);
  if (theeplusStepCut) theeplusStepCut->SetMaxStep(MaxChargedStep);
  if (themuStepCut) themuStepCut->SetMaxStep(MaxChargedStep);
  if (thehadronStepCut) thehadronStepCut->SetMaxStep(MaxChargedStep);

  if ( (MaxChargedStep != DBL_MAX) && (verboseLevel>1) )
    G4cout << "IcaPhysicsList: MaxChargedStep = " << MaxChargedStep/mm << " mm" << G4endl;
}

////////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetMaxNumOfCerenkovPhotonsPerStep(G4int newValue)
{
  MaxNumOfCerenkovPhotonsPerStep = newValue;
  if (theCerenkovProcess)
    theCerenkovProcess->SetMaxNumPhotonsPerStep(MaxNumOfCerenkovPhotonsPerStep);

  if (verboseLevel>1)
    G4cout << "IcaPhysicsList: MaxNumOfCerenkovPhotonsPerStep = " << MaxNumOfCerenkovPhotonsPerStep << G4endl;
}

////////////////////////////////////////////////////////////////////////////

void IcaPhysicsList::SetScintillationYieldFactor(G4double newValue)
{
  ScintillationYieldFactor = newValue;
  if (theScintillationProcess)
    theScintillationProcess->SetScintillationYieldFactor(ScintillationYieldFactor);

  if (verboseLevel>1)
    G4cout << "IcaPhysicsList: ScintillationYieldFactor = " << ScintillationYieldFactor << G4endl;
}

// end of file IcaPhysicsList.cc by Jacek M. Holeczek
